@@ -1,6 +1,6 @@
 /* mpfr_set_default_prec, mpfr_get_default_prec -- set/get default precision
 
-Copyright 1999-2001, 2004-2016 Free Software Foundation, Inc.
+Copyright 1999-2001, 2004-2017 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -23,15 +23,8 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 #include "mpfr-impl.h"
 
 /* default is IEEE double precision, i.e. 53 bits */
-MPFR_THREAD_ATTR mpfr_prec_t __gmpfr_default_fp_bit_precision \
-  = IEEE_DBL_MANT_DIG;
-
-#ifdef MPFR_WIN_THREAD_SAFE_DLL
-mpfr_prec_t * __gmpfr_default_fp_bit_precision_f()
-{
-  return &__gmpfr_default_fp_bit_precision;
-}
-#endif
+MPFR_THREAD_VAR (mpfr_prec_t, __gmpfr_default_fp_bit_precision,
+                 IEEE_DBL_MANT_DIG)
 
 void
 mpfr_set_default_prec (mpfr_prec_t prec)
